var workbook;
const excelJsConf = {
    colWidth: 12, //列宽
    rowHeight: 20, //行高
    colScale: 6.25, //列宽比例
    defaultFontColor: '#41464b', //默认字体颜色
    fonts:{ //字体
        0: '宋体（Song）',
        1: '微软雅黑',
        2: '黑体（ST Heiti）',
        3: '楷体（ST Kaiti）',
        4: '仿宋（ST FangSong）',
        5: '新宋体（ST Song）',
        6: '华文新魏',
        7: '华文行楷',
        8: '华文隶书',
        9: 'Arial',
        10: 'Times New Roman ',
        11: 'Tahoma ',
        12: 'Verdana',
    },
    border: { //边框
        styls: { //边框样式
            0: 'none',
            1: 'thin',
            2: 'hair',
            3: 'dotted',
            4: 'dashDot', // 'Dashed',
            5: 'dashDot',
            6: 'dashDotDot',
            7: 'double',
            8: 'medium',
            9: 'mediumDashed',
            10: 'mediumDashDot',
            11: 'mediumDashDotDot',
            12: 'slantDashDot',
            13: 'thick'
        },
        //边框种类
        types: ['border-left', 'border-right', 'border-top', 'border-bottom', 'border-all', 'border-outside', 'border-inside', 'border-horizontal', 'border-vertical'],
    },
    alignments: {
        vertical: {
            0: 'middle',
            1: 'top',
            2: 'bottom'
        },
        horizontal: {
            0: 'center',
            1: 'left',
            2: 'right',
        },
        wrapText: {
            0: false,
            1: false,
            2: true
        },
        textRotation: {
            0: 0,
            1: 45,
            2: -45,
            3: 'vertical',
            4: 90,
            5: -90
        }
    }
};

/**
 * 导出返回buffer
 */
async function exportXlsxBuffer(allSheets) {
    if (!allSheets || allSheets.length === 0) {
        return null;
    }
    workbook = new ExcelJS.Workbook();
    allSheets.every(function (sheet) {
        if (sheet.data.length === 0 || sheet.celldata.length === 0) return true;
        var showGridLines = parseInt(sheet.showGridLines) == 1 ? true : false;
        var worksheet = workbook.addWorksheet(sheet.name, {views: [{showGridLines: showGridLines}]});
        setStyleAndValue(sheet.data, worksheet);
        setAddImages((sheet.images || {}), sheet.zoomRatio, worksheet);
        setBorder(sheet.config.borderInfo, worksheet);
        setMerge(sheet.config.merge, worksheet);
        var rowAllLen = sheet.data.length;
        var colAllLen = rowAllLen > 0 ? sheet.data[0].length : 0;
        setRowAndColumnLen(sheet.config.rowlen, sheet.config.columnlen, rowAllLen, colAllLen, worksheet);
        setRowOrColHidden(sheet.config.rowhidden, sheet.config.colhidden, worksheet);
        return true
    });
    var buffer = await workbook.xlsx.writeBuffer();
    return buffer;
}

/**
 * 导出xlsx格式表
 */
async function exportXlsx(allSheets) {
    var buffer = await exportXlsxBuffer(allSheets);
    var base64 = bufferToBase64(buffer);
    return base64;
}

/**
 * 添加Sheet图片
 */
function setAddImages(images, zoomRatio, worksheet) {
    for(let key in images){
        var imageId = workbook.addImage({
            base64: images[key].src,
            extension: 'png',
        });
        var data = {
            tl: { col: images[key].columns[0], row: images[key].rows[0] },
            br: { col: images[key].columns[1] + 1, row: images[key].rows[1] + 1 },
            ext: { width: images[key].default.width, height: images[key].default.height },
            editAs: 'oneCell'
        };
        console.log(data);
        worksheet.addImage(imageId, data);
    }
}

//设置行与列隐藏
function setRowOrColHidden(rowhidden, colhidden, worksheet) {
    //设置隐藏行
    if (rowhidden) {
        for(var key in rowhidden){
            var row = worksheet.getRow(parseInt(key) + 1);
            row.hidden = true;
        }
    }
    //设置隐藏列
    if (colhidden) {
        for(var key in colhidden){
            var col = worksheet.getColumn(parseInt(key) + 1);
            col.hidden = true;
        }
    }
}

//设置行高与列宽
function setRowAndColumnLen(rowlen, columnlen, rowAllLen, colAllLen, worksheet) {
    //设置行高
    if (rowlen) {
        for(var key in rowlen){
            if (parseInt(key) >= rowAllLen) {
              break;
            }
            var row = worksheet.getRow(parseInt(key) + 1);
            row.height = rowlen[key];
        }
    }
    //设置列宽
    if (columnlen) {
        for(var key in columnlen){
            if (parseInt(key) >= colAllLen) {
                break;
            }
            var column = worksheet.getColumn(parseInt(key) + 1);
            column.width = columnlen[key] / excelJsConf.colScale;
        }
    }
}

//设置合并单元格
function setMerge(merge, worksheet) {
    if (merge) {
        var mergearr = Object.values(merge);
        mergearr.forEach(function (elem) {
            worksheet.mergeCells(elem.r + 1, elem.c + 1, elem.r + elem.rs, elem.c + elem.cs);
        })
    }
}

//设置边框样式
function setBorder(cellBorderArr, worksheet) {
    if (!Array.isArray(cellBorderArr)) return;
    cellBorderArr.forEach(function (border) {
        if (border.rangeType === 'range') {
            if (border.range && border.borderType && $.trim(border.style)) {
                var borderStyle = getBorderStyle(border.style, border.color);
                var rangeLen = border.range.length;
                if (rangeLen > 0) {
                    for (var ri = 0; ri < rangeLen; ri++) {
                        if (border.range[ri].row && border.range[ri].column) {
                            for (var i = border.range[ri].row[0]; i <= border.range[ri].row[1]; i++) {
                                for (var j = border.range[ri].column[0]; j <= border.range[ri].column[1]; j++) {
                                    var borderStyles = getRangeBorder(border.borderType, i, j, border.range[ri].row, border.range[ri].column, borderStyle, worksheet.getCell(i + 1, j + 1).border);
                                    worksheet.getCell(i + 1, j + 1).border = borderStyles;
                                }
                            }
                        }
                    }
                }
            }
        } else if (border.rangeType === 'cell') {
            if (border.value) {
                if ($.trim(border.value.col_index) && $.trim(border.value.row_index)) {
                    if (border.value.t || border.value.b || border.value.l || border.value.r) {
                        var borderStyles = getCellBorder(border.value.t, border.value.b, border.value.l, border.value.r, worksheet.getCell(border.value.row_index + 1, border.value.col_index + 1).border);
                        worksheet.getCell(border.value.row_index + 1, border.value.col_index + 1).border = borderStyles;
                    }
                }
            }
        }
    })
}

/**
 * 获取单元格边框
 */
function getCellBorder(t, b, l, r, borderStyles) {
    var border = borderStyles ? borderStyles : [];
    var borderStyle;
    if (t) {
        borderStyle = getBorderStyle(t.style, t.color);
        if (borderStyle) {
            border['top'] = borderStyle;
        }
    }
    if (b) {
        borderStyle = getBorderStyle(b.style, b.color);
        if (borderStyle) {
            border['bottom'] = borderStyle;
        }
    }
    if (l) {
        borderStyle = getBorderStyle(l.style, l.color);
        if (borderStyle) {
            border['left'] = borderStyle;
        }
    }
    if (r) {
        borderStyle = getBorderStyle(r.style, r.color);
        if (borderStyle) {
            border['right'] = borderStyle;
        }
    }
    return border;
}

//获取区域边框
function getRangeBorder(borderType = 'border-none', rowIndex = -1 , columnIndex = -1 , rowArr = [], columnArr = [] , borderStyle, borderStyles) {
    if (!borderType || borderType === 'border-none' || !$.trim(rowIndex) || rowIndex < 0 ||
        !$.trim(columnIndex) || columnIndex < 0 || !rowArr || rowArr.length === 0 ||
        !columnArr || columnArr.length === 0 || !borderStyle) {
        return null;
    }
    var border = borderStyles ? borderStyles : [];
    if (borderType === 'border-left' || borderType === 'border-all' || (borderType === 'border-outside' && columnIndex === columnArr[0])
        || ((borderType === 'border-inside'|| borderType === 'border-vertical') && columnIndex > columnArr[0])) {
        border['left'] = borderStyle;
    }
    if (borderStyle === 'border-right' || borderType === 'border-all' || (borderType === 'border-outside' && columnIndex === columnArr[1])
        || ((borderType === 'border-inside' || borderType === 'border-vertical') && (columnIndex < columnArr[1] || columnArr[0] === columnArr[1]))) {
        border['right'] = borderStyle;
    }
    if (borderStyle === 'border-top' || borderType === 'border-all' || (borderType === 'border-outside' && rowIndex === rowArr[0])
        || ((borderType === 'border-inside' || borderType === 'border-horizontal') && rowIndex > rowArr[0])) {
        border['top'] = borderStyle;
    }
    if (borderStyle === 'border-bottom' || borderType === 'border-all' || (borderType === 'border-outside' && rowIndex === rowArr[1])
        || ((borderType === 'border-inside' || borderType === 'border-horizontal') && (rowIndex < rowArr[1] || rowArr[0] === rowArr[1]))) {
        border['bottom'] = borderStyle;
    }
    return border;
}

//获取边框样式
function getBorderStyle(style = 0, color = excelJsConf.defaultFontColor) {
    if (style === 0) {
        return null;
    }
    if (color.trim().toUpperCase().indexOf("RGB") > -1) {
        color = colorRGBtoHex(color);
    }
    color = color.replace('#', '');
    if (color.length <= 4) {
        color = getExcelJsColor(color);
    }
    return {
        style: excelJsConf.border.styls[style],
        color: {argb: (color.length === 6 ? 'FF' : '') + color},
    };
}

// 设置单元格内容
function setStyleAndValue(cellArr, worksheet) {
    if (!Array.isArray(cellArr)) return;
    cellArr.forEach(function (row, rowIndex) {
        var rowR = worksheet.getRow(rowIndex + 1);
        rowR.height = excelJsConf.rowHeight;
        row.every(function (cell, cellIndex) {
            if (rowIndex === 0) {
                colR = worksheet.getColumn(cellIndex + 1);
                colR.width = excelJsConf.colWidth;
            }
            if (!cell) return true;
            // cell.f 单元函数，目前暂不使用函数 value = { formula: cell.f, result: cell.v };
            var value = $.trim(cell.v) ? cell.v : ($.trim(cell.m) ? cell.m : "");
            if ($.trim(cell.tb) && parseInt(cell.tb) === 2) {
                if (cell.ct && cell.ct.s && cell.ct.s.length > 0) {
                    value = "";
                    var len = cell.ct.s.length;
                    for(var i = 0; i < len; i++) {
                        value = value + cell.ct.s[i].v;
                    }
                }
            }
            var alignment = getAlignment(cell.vt, cell.ht, cell.tb, cell.rt, cell.tr);
            var fill = getFill(cell.bg);
            var font = getFont(cell.ff, cell.fc, cell.fs, cell.bl, cell.it, cell.cl, cell.un)
            var xlsxCell = worksheet.getCell(rowIndex + 1, cellIndex + 1);

            xlsxCell.alignment = alignment;
            if (fill) {
                xlsxCell.fill = fill;
            }
            xlsxCell.font = font;
            if ($.trim(value)) {
                xlsxCell.value = value + '';
            }
            return true;
        });
    });
}

//获取字体
function getFont(ff = 0, fc = excelJsConf.defaultFontColor, fs = 10, bl = 0, it = 0, cl = 0, un = 0) {
    if (fc.trim().toUpperCase().indexOf("RGB") > -1) {
        fc = colorRGBtoHex(fc);
    }
    fc = fc.replace('#', '');
    if (fc.length <= 4) {
        fc = getExcelJsColor(fc);
    }
    if (parseFloat(ff).toString() == "NaN") {
        ff = 0;
    }
    var font = {
        name: excelJsConf.fonts[ff],
        family: 1,
        size: fs,
        color: {argb: (fc.length === 6 ? 'FF' : '') + fc},
        bold: bl === 0 ? false : true,
        italic: it === 0 ? false : true,
        underline: un === 0 ? false : true,
        strike: cl === 0 ? false : true
    }
    return font
}

//获取单元格内容居中方式
function getAlignment(vt = 0, ht = 1, tb = 0, rt = -1000, tr = 0) {
    var alignment = {
        vertical: excelJsConf.alignments.vertical[vt],
        horizontal: excelJsConf.alignments.horizontal[ht],
        wrapText: excelJsConf.alignments.wrapText[tb],
        textRotation: rt > -1000 ? rt : excelJsConf.alignments.textRotation[tr]
    }
    return alignment
}

//获取填充背景颜色
function getFill(bg) {
    if (!bg) {
        return null;
    }
    if (bg.trim().toUpperCase().indexOf("RGB") > -1) {
        bg = colorRGBtoHex(bg);
    }
    if (!bg) {
        return null;
    }
    bg = bg.replace('#', '');
    if (bg.length <= 4) {
        bg = getExcelJsColor(bg);
    }
    var fill = {
        type: 'pattern',
        pattern: 'solid',
        fgColor: {argb: (bg.length === 6 ? 'FF' : '') + bg}
    }
    return fill
}
